@@ -14,7 +14,8 @@
  * Lesser General Public License for more details.
  *
  * You should have received a copy of the GNU Lesser General Public
- * License along with this program; if not, see <https://www.gnu.org/licenses/>.
+ * License along with this program; if not, see <https://gnu.org/licenses/>.
+ * SPDX-License-Identifier: LGPL-2.1-or-later
  */
 
 /*
@@ -96,6 +97,7 @@ struct engine_llass
     int gpg_agent:1;  /* Assume this is a gpg-agent connection.  */
   } opt;
 
+  char request_origin[10];  /* Copy from the CTX.  */
 };
 typedef struct engine_llass *engine_llass_t;
 
@@ -265,7 +267,7 @@ llass_new (void **engine, const char *file_name, const char *home_dir,
         goto leave;
       if (dft_display)
         {
-          if (asprintf (&optstr, "OPTION display=%s", dft_display) < 0)
+          if (gpgrt_asprintf (&optstr, "OPTION display=%s", dft_display) < 0)
             {
               err = gpg_error_from_syserror ();
               free (dft_display);
@@ -275,7 +277,7 @@ llass_new (void **engine, const char *file_name, const char *home_dir,
 
           err = assuan_transact (llass->assuan_ctx, optstr, NULL, NULL, NULL,
                                  NULL, NULL, NULL);
-          free (optstr);
+          gpgrt_free (optstr);
           if (err)
             goto leave;
         }
@@ -304,14 +306,14 @@ llass_new (void **engine, const char *file_name, const char *home_dir,
 	 ways, e.g., when /dev/pts is not accessible under chroot.  */
       if (!rc)
 	{
-	  if (asprintf (&optstr, "OPTION ttyname=%s", dft_ttyname) < 0)
+	  if (gpgrt_asprintf (&optstr, "OPTION ttyname=%s", dft_ttyname) < 0)
 	    {
 	      err = gpg_error_from_syserror ();
 	      goto leave;
 	    }
 	  err = assuan_transact (llass->assuan_ctx, optstr, NULL, NULL, NULL,
 				 NULL, NULL, NULL);
-	  free (optstr);
+	  gpgrt_free (optstr);
 	  if (err)
             goto leave;
 
@@ -320,7 +322,7 @@ llass_new (void **engine, const char *file_name, const char *home_dir,
 	    goto leave;
 	  if (dft_ttytype)
 	    {
-	      if (asprintf (&optstr, "OPTION ttytype=%s", dft_ttytype) < 0)
+	      if (gpgrt_asprintf (&optstr, "OPTION ttytype=%s", dft_ttytype)< 0)
 		{
 		  err = gpg_error_from_syserror ();
 		  free (dft_ttytype);
@@ -330,7 +332,7 @@ llass_new (void **engine, const char *file_name, const char *home_dir,
 
 	      err = assuan_transact (llass->assuan_ctx, optstr, NULL, NULL,
 				     NULL, NULL, NULL, NULL);
-	      free (optstr);
+	      gpgrt_free (optstr);
 	      if (err)
                 goto leave;
 	    }
@@ -365,6 +367,24 @@ llass_new (void **engine, const char *file_name, const char *home_dir,
 }
 
 
+/* Copy flags from CTX into the engine object.  */
+static void
+llass_set_engine_flags (void *engine, const gpgme_ctx_t ctx)
+{
+  engine_llass_t llass = engine;
+
+  if (ctx->request_origin)
+    {
+      if (strlen (ctx->request_origin) + 1 > sizeof llass->request_origin)
+        strcpy (llass->request_origin, "xxx"); /* Too long  - force error */
+      else
+        strcpy (llass->request_origin, ctx->request_origin);
+    }
+  else
+    *llass->request_origin = 0;
+}
+
+
 static gpgme_error_t
 llass_set_locale (void *engine, int category, const char *value)
 {
@@ -408,13 +428,13 @@ llass_set_locale (void *engine, int category, const char *value)
   if (!value)
     return 0;
 
-  if (asprintf (&optstr, "OPTION %s=%s", catstr, value) < 0)
+  if (gpgrt_asprintf (&optstr, "OPTION %s=%s", catstr, value) < 0)
     err = gpg_error_from_syserror ();
   else
     {
       err = assuan_transact (llass->assuan_ctx, optstr, NULL, NULL,
 			     NULL, NULL, NULL, NULL);
-      free (optstr);
+      gpgrt_free (optstr);
     }
   return err;
 }
@@ -473,13 +493,13 @@ llass_status_handler (void *opaque, int fd)
 	     case, we are done for now.  */
 	  if (gpg_err_code (err) == GPG_ERR_EAGAIN)
 	    {
-	      TRACE1 (DEBUG_CTX, "gpgme:llass_status_handler", llass,
+	      TRACE (DEBUG_CTX, "gpgme:llass_status_handler", llass,
 		      "fd 0x%x: EAGAIN reading assuan line (ignored)", fd);
 	      err = 0;
 	      continue;
 	    }
 
-	  TRACE2 (DEBUG_CTX, "gpgme:llass_status_handler", llass,
+	  TRACE (DEBUG_CTX, "gpgme:llass_status_handler", llass,
 		  "fd 0x%x: error reading assuan line: %s",
                   fd, gpg_strerror (err));
 	}
@@ -510,7 +530,7 @@ llass_status_handler (void *opaque, int fd)
             err = llass->user.data_cb (llass->user.data_cb_value,
                                        src, linelen);
 
-          TRACE2 (DEBUG_CTX, "gpgme:llass_status_handler", llass,
+          TRACE (DEBUG_CTX, "gpgme:llass_status_handler", llass,
 		  "fd 0x%x: D inlinedata; status from cb: %s",
                   fd, (llass->user.data_cb ?
                        (err? gpg_strerror (err):"ok"):"no callback"));
@@ -523,7 +543,7 @@ llass_status_handler (void *opaque, int fd)
           if (llass->user.data_cb)
             err = llass->user.data_cb (llass->user.data_cb_value, NULL, 0);
 
-          TRACE2 (DEBUG_CTX, "gpgme:llass_status_handler", llass,
+          TRACE (DEBUG_CTX, "gpgme:llass_status_handler", llass,
 		  "fd 0x%x: END line; status from cb: %s",
                   fd, (llass->user.data_cb ?
                        (err? gpg_strerror (err):"ok"):"no callback"));
@@ -549,7 +569,7 @@ llass_status_handler (void *opaque, int fd)
             err = llass->user.status_cb (llass->user.status_cb_value,
                                          src, args);
 
-          TRACE3 (DEBUG_CTX, "gpgme:llass_status_handler", llass,
+          TRACE (DEBUG_CTX, "gpgme:llass_status_handler", llass,
 		  "fd 0x%x: S line (%s) - status from cb: %s",
                   fd, line+2, (llass->user.status_cb ?
                                (err? gpg_strerror (err):"ok"):"no callback"));
@@ -595,7 +615,7 @@ llass_status_handler (void *opaque, int fd)
 	    err = atoi (line+4);
 	  else
 	    err = gpg_error (GPG_ERR_GENERAL);
-          TRACE2 (DEBUG_CTX, "gpgme:llass_status_handler", llass,
+          TRACE (DEBUG_CTX, "gpgme:llass_status_handler", llass,
 		  "fd 0x%x: ERR line: %s",
                   fd, err ? gpg_strerror (err) : "ok");
 
@@ -612,7 +632,7 @@ llass_status_handler (void *opaque, int fd)
 	       && line[0] == 'O' && line[1] == 'K'
 	       && (line[2] == '\0' || line[2] == ' '))
 	{
-          TRACE1 (DEBUG_CTX, "gpgme:llass_status_handler", llass,
+          TRACE (DEBUG_CTX, "gpgme:llass_status_handler", llass,
 		  "fd 0x%x: OK line", fd);
 
 	  llass->last_op_err = 0;
@@ -637,7 +657,7 @@ add_io_cb (engine_llass_t llass, iocb_data_t *iocbd, gpgme_io_cb_t handler)
 {
   gpgme_error_t err;
 
-  TRACE_BEG2 (DEBUG_ENGINE, "engine-assuan:add_io_cb", llass,
+  TRACE_BEG  (DEBUG_ENGINE, "engine-assuan:add_io_cb", llass,
               "fd %d, dir %d", iocbd->fd, iocbd->dir);
   err = (*llass->io_cbs.add) (llass->io_cbs.add_priv,
 			      iocbd->fd, iocbd->dir,
@@ -660,6 +680,21 @@ start (engine_llass_t llass, const char *command)
   int nfds;
   int i;
 
+  if (*llass->request_origin && llass->opt.gpg_agent)
+    {
+      char *cmd;
+
+      cmd = _gpgme_strconcat ("OPTION pretend-request-origin=",
+                              llass->request_origin, NULL);
+      if (!cmd)
+        return gpg_error_from_syserror ();
+      err = assuan_transact (llass->assuan_ctx, cmd, NULL, NULL, NULL,
+                             NULL, NULL, NULL);
+      free (cmd);
+      if (err && gpg_err_code (err) != GPG_ERR_UNKNOWN_OPTION)
+        return err;
+    }
+
   /* We need to know the fd used by assuan for reads.  We do this by
      using the assumption that the first returned fd from
      assuan_get_active_fds() is always this one.  */
@@ -749,7 +784,7 @@ llass_io_event (void *engine, gpgme_event_io_t type, void *type_data)
 {
   engine_llass_t llass = engine;
 
-  TRACE3 (DEBUG_ENGINE, "gpgme:llass_io_event", llass,
+  TRACE (DEBUG_ENGINE, "gpgme:llass_io_event", llass,
           "event %p, type %d, type_data %p",
           llass->io_cbs.event, type, type_data);
   if (llass->io_cbs.event)
@@ -775,8 +810,8 @@ struct engine_ops _gpgme_engine_ops_assuan =
     NULL,               /* set_colon_line_handler */
     llass_set_locale,
     NULL,		/* set_protocol */
+    llass_set_engine_flags,
     NULL,               /* decrypt */
-    NULL,               /* decrypt_verify */
     NULL,               /* delete */
     NULL,		/* edit */
     NULL,               /* encrypt */
@@ -787,6 +822,7 @@ struct engine_ops _gpgme_engine_ops_assuan =
     NULL,               /* import */
     NULL,               /* keylist */
     NULL,               /* keylist_ext */
+    NULL,               /* keylist_data */
     NULL,               /* keysign */
     NULL,               /* tofu_policy */
     NULL,               /* sign */
@@ -796,6 +832,7 @@ struct engine_ops _gpgme_engine_ops_assuan =
     llass_transact,     /* opassuan_transact */
     NULL,		/* conf_load */
     NULL,		/* conf_save */
+    NULL,		/* conf_dir */
     NULL,               /* query_swdb */
     llass_set_io_cbs,
     llass_io_event,
